var winterTimeOffsetSec;
var summerTimeOffsetSec;
var summerTimeStartDay;
var summerTimeStartHour;
var summerTimeStartMonth;
var summerTimeStartMinute;
var summerTimeStartDayOfWeek;
var summerTimeEndDay;
var summerTimeEndHour;
var summerTimeEndMonth;
var summerTimeEndMinute;
var summerTimeEndDayOfWeek;
const summerTimeStartDatesCache = {};
const summerTimeEndDatesCache = {};

const parseTimezoneParams = (rawData, callBack) => {
    const receivedData = JSON.parse(rawData);
    winterTimeOffsetSec = Number.parseInt(receivedData["_TIME_ZONE"]);
    summerTimeOffsetSec = winterTimeOffsetSec + Number.parseInt(receivedData["_STIME"]);
    summerTimeStartDay = Number.parseInt(receivedData["_SDAY"]);
    summerTimeStartHour = Number.parseInt(receivedData["_SHOUR"]);
    summerTimeStartMonth = Number.parseInt(receivedData["_SMON"]);
    summerTimeStartMinute = Number.parseInt(receivedData["_SMIN"]);
    summerTimeStartDayOfWeek = Number.parseInt(receivedData["_SDOW"]);
    summerTimeEndDay = Number.parseInt(receivedData["_EDAY"]);
    summerTimeEndHour = Number.parseInt(receivedData["_EHOUR"]);
    summerTimeEndMonth = Number.parseInt(receivedData["_EMON"]);
    summerTimeEndMinute = Number.parseInt(receivedData["_EMIN"]);
    summerTimeEndDayOfWeek = Number.parseInt(receivedData["_EDOW"]);
    callBack();
};

const initTimeZoneParameters = (callback) => {
    $.get("/lib/get_timezone_parameters.html", rawData => parseTimezoneParams(rawData, callback));
};

const getTimezoneChangeDate = (year, month, day, hour, minute, dayOfWeek, offsetSeconds) => {
    var dayOffsetCalculationDate = new Date(Date.UTC(year, month, day, hour, minute));
    var dayOffset = 0;
    while (dayOffsetCalculationDate.getUTCDay() !== summerTimeEndDayOfWeek) {
        dayOffset++;
        dayOffsetCalculationDate = new Date(Date.UTC(
            year, month, day + dayOffset, hour, minute));
    }
    return new Date(Date.UTC(
        year, month, day + dayOffset, hour, minute, -offsetSeconds));
};

const getSummerTimeStartDate = year => {
    const yearKey = `year${year}`;
    if (!summerTimeStartDatesCache.hasOwnProperty(yearKey))
        summerTimeStartDatesCache[yearKey] = getTimezoneChangeDate(year, summerTimeStartMonth - 1, summerTimeStartDay,
            summerTimeStartHour, summerTimeStartMinute, summerTimeStartDayOfWeek, winterTimeOffsetSec);
    return summerTimeStartDatesCache[yearKey];
};

const getSummerTimeEndDate = year => {
    const yearKey = `year${year}`;
    if (!summerTimeEndDatesCache.hasOwnProperty(yearKey))
        summerTimeEndDatesCache[yearKey] = getTimezoneChangeDate(year, summerTimeEndMonth - 1, summerTimeEndDay, summerTimeEndHour, summerTimeEndMinute,
            summerTimeEndDayOfWeek, summerTimeOffsetSec);
    return summerTimeEndDatesCache[yearKey];
};

const getFakedLocalDeviceTime = seconds => {
    const utcMillis = seconds * 1000;
    const date = new Date(utcMillis);
    const year = date.getUTCFullYear();
    const isSummerTime = utcMillis >= getSummerTimeStartDate(year).getTime() && utcMillis < getSummerTimeEndDate(year).getTime();
    return new Date(utcMillis + (isSummerTime ? summerTimeOffsetSec * 1000 : winterTimeOffsetSec * 1000) + (date.getTimezoneOffset() * 60000));
};

const toDeviceTimeString = seconds => {
    return deviceTimeStringInternal(seconds, "HH:MM:ss.l");
};

const toDeviceTimeStringNoMs = seconds => {
    const fakedLocalDeviceTime = getFakedLocalDeviceTime(seconds);
    return fakedLocalDeviceTime.toLocaleDateString([], {year: 'numeric', month: '2-digit', day:'2-digit'})
        + ", " + fakedLocalDeviceTime.toLocaleTimeString([], {hour: '2-digit', minute:'2-digit', second:'2-digit'});
};

const deviceTimeStringInternal = (seconds, format) => {
    const fakedLocalDeviceTime = getFakedLocalDeviceTime(seconds);
    return fakedLocalDeviceTime.toLocaleDateString([], {year: 'numeric', month: '2-digit', day:'2-digit'})
        + ", " + dateFormat(fakedLocalDeviceTime, format);
};

const setActualDeviceTime = () => {
    getInternalUTCAsSeconds(utcAsSeconds => {
        const fakedLocalDeviceTime = getFakedLocalDeviceTime(utcAsSeconds);
        $('#time').text(fakedLocalDeviceTime.toLocaleTimeString([], {hour: '2-digit', minute:'2-digit'})
            + " / " + fakedLocalDeviceTime.toLocaleDateString([], {year: 'numeric', month: '2-digit', day:'2-digit'}));
    });
};

const getActualDeviceTimeFakedToLocalTime = (callback) => {
    getInternalUTCAsSeconds(utcAsSeconds => callback(getFakedLocalDeviceTime(utcAsSeconds)));
};

const setActualDeviceTimeRepeatedly = () => {
    setActualDeviceTime();
    setInterval(function () {
        setActualDeviceTime();
    }, 3000);
};


const getInternalUTCAsSeconds = (callback) => {
    $.getJSON("/lib/get_time_info.html", timeInfo => callback(timeInfo["_UTCTIME"][0]));
};
