/*
TRANSIENT REASONS:
FLAG HEX   - FLAG decimal - Reason               - I18n
0x00000001 -            1 - CURRENT_ABS_FLAG     - transient.current
0x00000010 -           16 - VOLTAGE_ABS_FLAG     - transient.voltage
0x00000100 -          256 - undefined            - transient.reasonundefined
0x00001000 -         4096 - EVENT_FLAG           - transient.event
0x00004000 -        16384 - VOLTAGE_LL_FLAG      - transient.voltage
0x00010000 -        65536 - VOLTAGE_DELTA_FLAG   - transient.voltagedelta
0x00100000 -      1048576 - undefined            - transient.reasonundefined
0x01000000 -     16777216 - VOLTAGE_ENVELOV_FLAG - transient.voltageenvelope
0x10000000 -    268435456 - USE_ENUM_FLAG

ENUMFLAG - Trigger                           - status           - 
      -1 - HWW_NONE                          - no public use    - transient.triggerundefined
       0 - HWW_TRIGGER_KW_HIGH               - not in use (yet) - transient.triggerundefined
       1 - HWW_TRIGGER_KW_HIGH_END           - not in use (yet) - transient.triggerundefined
       2 - HWW_TRIGGER_KVA_HIGH              - not in use (yet) - transient.triggerundefined
       3 - HWW_TRIGGER_KVA_HIGH_END          - not in use (yet) - transient.triggerundefined
       4 - HWW_TRIGGER_KW_SUM_HIGH           - in use           - transient.kwhigh
       5 - HWW_TRIGGER_KW_SUM_HIGH_END       - in use           - transient.kwhighend
       6 - HWW_TRIGGER_KVA_SUM_HIGH          - in use           - transient.kvahigh
       7 - HWW_TRIGGER_KVA_SUM_HIGH_END      - in use           - transient.kvahighend
       8 - HWW_TRIGGER_U_UNBALANCE           - in use           - transient.voltageunbalance
       9 - HWW_TRIGGER_U_UNBALANCE_END       - in use           - transient.voltageunbalanceend
      10 - HWW_TRIGGER_U_THD                 - in use           - transient.voltagethd
      11 - HWW_TRIGGER_U_THD_END             - in use           - transient.voltagethdend
      12 - HWW_TRIGGER_COUNT                 - no public use    - transient.triggerundefined
      13 -                                   - undefined        - transient.triggerundefined
      14 -                                   - undefined        - transient.triggerundefined
      15 -                                   - undefined        - transient.triggerundefined
*/

const TRIGGER_ENUM_FLAG = Object.freeze({"kwhigh": "transient.kwhigh", "kwhighEnd": "transient.kwhighend", "kvahigh": "transient.kvahigh",
    "kvahighEnd": "transient.kvahighend", "voltageunbalance": "transient.voltageunbalance", "voltageunbalanceEnd": "transient.voltageunbalanceend",
    "voltagethd": "transient.voltagethd", "voltagethdEnd": "transient.voltagethdend", "undefined": "transient.triggerundefined"});
// Relevant enum flag values without voltage thd because it is not a sum value
const enumFlagSumValues = [TRIGGER_ENUM_FLAG.kwhigh, TRIGGER_ENUM_FLAG.kwhighEnd, TRIGGER_ENUM_FLAG.kvahigh, 
    TRIGGER_ENUM_FLAG.kvahighEnd, TRIGGER_ENUM_FLAG.voltageunbalance, TRIGGER_ENUM_FLAG.voltageunbalanceEnd];

const TRANSIENT_REASONS = Object.freeze({"currentAbs" : "transient.current" , "voltageAbs" : "transient.voltage" , 
    "event" : "transient.event" , "voltageDelta" : "transient.voltagedelta" , "voltageEnvelope" : "transient.voltageenvelope" , 
    "undefined" : "transient.reasonundefined"});

var COUNTER_POLL_INTERVAL_MS = 15000;

REASON_UMAX = 1;
REASON_UMIN = 2;
REASON_IMAX = 3;
REASON_UOFF = 4;
var counter = null;

var SITE = "TRNS";

var SelectedTransElement = null;

let graph = null;

let selectedTransientIndex = -1;

let graphLoading = false;

/**
 * xPhone doesn't update scrolling div properly without this fix.
 */
const initTransientsPage = () => {
    graph = new TransientGraph('transientsgraph');
    $('#colorboxUL1').css("background-color", GRAPH_OPTIONS.colors[CHANNELS.UL1 - 1]);
    $('#colorboxUL2').css("background-color", GRAPH_OPTIONS.colors[CHANNELS.UL2 - 1]);
    $('#colorboxUL3').css("background-color", GRAPH_OPTIONS.colors[CHANNELS.UL3 - 1]);
    $('#colorboxUL4').css("background-color", GRAPH_OPTIONS.colors[CHANNELS.UL4 - 1]);
    $('#colorboxIL1').css("background-color", GRAPH_OPTIONS.colors[CHANNELS.IL1 - 1]);
    $('#colorboxIL2').css("background-color", GRAPH_OPTIONS.colors[CHANNELS.IL2 - 1]);
    $('#colorboxIL3').css("background-color", GRAPH_OPTIONS.colors[CHANNELS.IL3 - 1]);
    $('#colorboxIL4').css("background-color", GRAPH_OPTIONS.colors[CHANNELS.IL4 - 1]);
    addOnChangeListener($('#UL1'), $('#colorboxUL1'), CHANNELS.UL1);
    addOnChangeListener($('#UL2'), $('#colorboxUL2'), CHANNELS.UL2);
    addOnChangeListener($('#UL3'), $('#colorboxUL3'), CHANNELS.UL3);
    addOnChangeListener($('#UL4'), $('#colorboxUL4'), CHANNELS.UL4);
    addOnChangeListener($('#IL1'), $('#colorboxIL1'), CHANNELS.IL1);
    addOnChangeListener($('#IL2'), $('#colorboxIL2'), CHANNELS.IL2);
    addOnChangeListener($('#IL3'), $('#colorboxIL3'), CHANNELS.IL3);
    addOnChangeListener($('#IL4'), $('#colorboxIL4'), CHANNELS.IL4);
    $('#clearzoom').on("click", () => {
        graph.resetZoomState();
        graph.redraw();
    });
};

const phaseToULChannel = [1, 3, 5, 7];
const phaseToILChannel = [2, 4, 6, 8];

const addOnChangeListener = (checkbox, colorbox, channel) => {
    checkbox.on("change", () => {
        if (selectedTransientIndex !== -1) {
            if (checkbox.is(':checked')) {
                setLoading();
                const index = startIndex(trnsdata[selectedTransientIndex]["data_cnt"], trnsdata[selectedTransientIndex].pre);
                $.getJSON("../../lib/transients/transient_data_jasic.html?_transientindex=" + selectedTransientIndex
                    + "&_transientchannel=" + channel + "&_transientstartindex=" + index, (data) => {
                    if (data && data.data)
                        graph.addDataForChannel(channel, data.data, trnsdata[selectedTransientIndex].StartTime,
                            trnsdata[selectedTransientIndex]["s_freq"], trnsdata[selectedTransientIndex].pre - index);
                }).always(() => {
                    colorbox.css("visibility", "visible");
                    graph.redraw();
                    setLoadingFinished();
                });
            } else {
                colorbox.css("visibility", "hidden");
                graph.clearDataForChannel(channel);
                graph.redraw();
            }
        }
    });
};

const mapFlagsToReasonOfTransient = (transientReasonFlag, triggerEnumFlag) => {
    if (bittest(transientReasonFlag, 28))
        return evaluateTriggerEnumFlag(triggerEnumFlag);
    else
        return evaluateTransientReasonFlag(transientReasonFlag);
};

const evaluateTriggerEnumFlag = (triggerEnumFlag) => {
    triggerEnumFlag.toFixed(0);
    switch(triggerEnumFlag) {
        case 4:
            return TRIGGER_ENUM_FLAG.kwhigh;
        case 5:
            return TRIGGER_ENUM_FLAG.kwhighEnd;
        case 6:
            return TRIGGER_ENUM_FLAG.kvahigh;
        case 7:
            return TRIGGER_ENUM_FLAG.kvahighEnd;
        case 8:
            return TRIGGER_ENUM_FLAG.voltageunbalance;
        case 9:
            return TRIGGER_ENUM_FLAG.voltageunbalanceEnd;
        case 10:
            return TRIGGER_ENUM_FLAG.voltagethd;
        case 11:
            return TRIGGER_ENUM_FLAG.voltagethdEnd;
        default:
            return TRIGGER_ENUM_FLAG.undefined;
    }
};

const evaluateTransientReasonFlag = (transientReasonFlag) => {
    if (bittest(transientReasonFlag, 24))
        return TRANSIENT_REASONS.voltageEnvelope;
    else if (bittest(transientReasonFlag, 20))
        return TRANSIENT_REASONS.undefined;
    else if (bittest(transientReasonFlag, 16))
        return TRANSIENT_REASONS.voltageDelta;
    else if (bittest(transientReasonFlag, 12))
        return TRANSIENT_REASONS.event;
    else if (bittest(transientReasonFlag, 8))
        return TRANSIENT_REASONS.undefined;
    else if (bittest(transientReasonFlag, 4))
        return TRANSIENT_REASONS.voltageAbs;
    else if (bittest(transientReasonFlag, 0))
        return TRANSIENT_REASONS.currentAbs;
    else
        return TRANSIENT_REASONS.undefined;
};

const setLoading = () => {
    $("#loader").show();
    graphLoading = true;
    enableDIsableCheckboxes(false);
};

const setLoadingFinished = () => {
    graphLoading = false;
    $("#loader").hide();
    enableDIsableCheckboxes(true);
};

function appliedPhoneRedrawFix() {
    addStyleClass(ById("list"), "ui-force-redraw");
    window.setTimeout(function () {
        removeStyleClass(ById("list"), "ui-force-redraw");
    }, 100);
}

function fillspan(id, txt) {
    var obj = document.getElementById(id);
    if (obj)
        obj.innerHTML = txt;
}

function translist_click(evt) {
    if (graphLoading) {
        alert(tl("transients.wait"));
    } else {
        evt = (evt) ? evt : ((window.event) ? window.event : "");
        var elem = (evt.target) ? evt.target : evt.srcElement;

        var watchdog = 10;
        while (elem.tagName != "DIV" || !(elem.className && elem.className.indexOf("transitem") >= 0)) {
            elem = elem.parentNode;
            if (watchdog-- == 0)
                break;
        }

        var newlySelectedTransientIndex = parseInt(elem.getAttribute("Name"));
        if (newlySelectedTransientIndex !== selectedTransientIndex) {
            if (SelectedTransElement)
                removeStyleClass(SelectedTransElement, "transitem_selected");

            selectedTransientIndex = newlySelectedTransientIndex;
            resetChannelCheckboxes();
            const transientPhase = trnsdata[selectedTransientIndex].phase + 1;
            let channels = [phaseToULChannel[transientPhase - 1]];
            if (trnsdata[selectedTransientIndex].reason == TRIGGER_ENUM_FLAG.voltageunbalance || trnsdata[selectedTransientIndex].reason == TRIGGER_ENUM_FLAG.voltageunbalanceEnd) {
                channels = [1, 3, 5];
            } else if (trnsdata[selectedTransientIndex].reason == TRIGGER_ENUM_FLAG.kwhigh || trnsdata[selectedTransientIndex].reason == TRIGGER_ENUM_FLAG.kwhighEnd) {
                channels = [1, 2, 3, 4, 5, 6];
            } else if (trnsdata[selectedTransientIndex].reason == TRIGGER_ENUM_FLAG.kvahigh || trnsdata[selectedTransientIndex].reason == TRIGGER_ENUM_FLAG.kvahighEnd) {
                channels = [1, 2, 3, 4, 5, 6];
            } else if (bittest(trnsdata[selectedTransientIndex].flag, 0)) {
                channels = [phaseToILChannel[transientPhase - 1]];
            }
            const index = startIndex(trnsdata[selectedTransientIndex]["data_cnt"], trnsdata[selectedTransientIndex].pre);
            setLoading();
            graph.clearData();
            loadChannels(channels, index);
            fillspan("info_starttime", toDeviceTimeString(trnsdata[selectedTransientIndex].StartTime / 1000));
            fillspan("info_phase", phaseOfTransient(selectedTransientIndex));
            fillspan("info_reason", tl(trnsdata[selectedTransientIndex].reason));
            fillspan("info_pre", trnsdata[selectedTransientIndex].pre);
            fillspan("info_data_cnt", trnsdata[selectedTransientIndex].data_cnt);
            fillspan("info_sfreq", trnsdata[selectedTransientIndex].s_freq + " Hz");
            if (bittest(trnsdata[selectedTransientIndex].flag, 14)) {
                $("#UL1").next("span").text(tl('_ULL[0]'));
                $("#UL2").next("span").text(tl('_ULL[1]'));
                $("#UL3").next("span").text(tl('_ULL[2]'));
                if (trnsdata[selectedTransientIndex].phase === 3) {
                    $("#UL4").show();
                    $("#UL4").next("span").show();
                } else {
                    $("#UL4").hide();
                    $("#UL4").next("span").hide();
                }
            } else {
                $("#UL1").next("span").text(tl('_ULN[0]'));
                $("#UL2").next("span").text(tl('_ULN[1]'));
                $("#UL3").next("span").text(tl('_ULN[2]'));
                $("#UL4").show();
                $("#UL4").next("span").show();
            }
            addStyleClass(elem, "transitem_selected");
            if (is_phone)
                appliedPhoneRedrawFix();
            SelectedTransElement = elem;
        }
    }
}

const loadChannels = (channels, index) => {
    const channel = channels[0];
    const newChannels = channels.slice(1);
    $.getJSON("../../lib/transients/transient_data_jasic.html?_transientindex=" + selectedTransientIndex +
        "&_transientchannel=" + channel + "&_transientstartindex=" + index, (data) => {
        if (data && data.data) {
            graph.addDataForChannel(channel, data.data, trnsdata[selectedTransientIndex].StartTime,
                trnsdata[selectedTransientIndex]["s_freq"], trnsdata[selectedTransientIndex].pre - index);
            setCheckboxForChannel(channel, true);
        }
        if (newChannels.length > 0) {
            loadChannels(newChannels, index);
        } else {
            graph.resetZoomState();
            graph.redraw();
            setLoadingFinished();
        }
    }).fail(() => {
        graph.resetZoomState();
        graph.redraw();
        setLoadingFinished();
    });
}

const phaseOfTransient = (index) => {
    if ((bittest(trnsdata[index].flag, 14) && bittest(trnsdata[index].flag, 28) && trnsdata[index].enumflag >= 8 && trnsdata[index].enumflag <= 9)) {
        return "";
    } else if ((bittest(trnsdata[index].flag, 14) && bittest(trnsdata[index].flag, 4)) ||
        (bittest(trnsdata[index].flag, 14) && bittest(trnsdata[index].flag, 16)) ||
        (bittest(trnsdata[index].flag, 14) && bittest(trnsdata[index].flag, 24)) ||
        (bittest(trnsdata[index].flag, 14) && bittest(trnsdata[index].flag, 28) && trnsdata[index].enumflag >= 10 && trnsdata[index].enumflag <= 11)) {
        if (trnsdata[index].phase === 0) {
            return "L1-L2"
        } else if (trnsdata[index].phase === 1) {
            return "L2-L3"
        } else if (trnsdata[index].phase === 2) {
            return "L3-L1"
        } else {
            return "L4";
        }
    } else {
        return "L" + (enumFlagSumValues.indexOf(trnsdata[index].reason) > -1 ? "1..L3" : (trnsdata[index].phase + 1));
    }
}

const startIndex = (count, pre) => {
    const minimumPretriggerIfCountLow = 6000;
    const minimumCountIfPretriggerLow = 12000;

    if (pre <= minimumPretriggerIfCountLow || count <= minimumCountIfPretriggerLow)
        return 0;
    const indexPreMinusMinimum = pre - minimumPretriggerIfCountLow;
    const indexCountMinusMinimum = count - (minimumCountIfPretriggerLow + 1);
    return indexPreMinusMinimum < indexCountMinusMinimum ? indexPreMinusMinimum : indexCountMinusMinimum;
};

function DataReceive(text) {
    var data = text.split("-NEXT_ARRAY-");
    trnsdata = eval("(" + data[0] + ")");
    counter = eval("(" + data[1] + ")");

    appendReasonToTransData(trnsdata);

    var SelectElement = 0;
    if (/show=([0-9]+)/.exec(document.location.search))
        SelectElement = parseInt(RegExp.$1);
    var parent = document.getElementById("list");

    if (trnsdata.length == 0) {
        var el = document.createElement("div");
        el.className = "transitem" + ((i % 2 == 0) ? "even" : "");
        el.setAttribute("name", i);
        var s = "<div style=\"float:right;font-size:12pt;color:#909090;padding:3px\">" + tl("menu.transient") + ": 0" + "</b><br>";
        el.innerHTML = s;
        parent.appendChild(el);
    }
    for (var i in trnsdata) {
        if (i in Array.prototype)
            continue;
        var el = document.createElement("div");
        el.className = "transitem" + ((i % 2 == 0) ? "even" : "");
        el.setAttribute("name", i);
        const currentTransient = trnsdata[i];
        const phase = phaseOfTransient(i);
        var s = "<div style=\"float:right;font-size:10pt;color:#C0C0C0;padding:3px\">" + (parseInt(i) + 1) + "</div><b style=\"cursor:pointer\">" + 
            tl(trnsdata[i].reason) + " " + phase + "</b><br>";
        s += toDeviceTimeString(trnsdata[i].StartTime / 1000);
        el.innerHTML = s;
        el.onclick = translist_click;
        parent.appendChild(el);

        if (i == SelectElement) {
            // Fake Event in order to populate Table Info on startup
            fakeevent = {'target': el};
            translist_click(fakeevent);
        }

    }

    if (is_phone) {
        if (iListScroll != null) {
            iListScroll.position = 0;
            iListScroll.refresh();
        }
    }

}

const appendReasonToTransData = (transients) => {
    for (var i in transients) {
        let reason = mapFlagsToReasonOfTransient(transients[i].flag , transients[i].enumflag);
        transients[i].reason = reason;
    }
};

function poll_counters() {
    $.getJSON("json.do?_evt_count,_trans_count", data => {
        var obj = eval(data);
        if (counter["_trans_count"] != obj["_TRANS_COUNT"][0] && SITE == "TRNS") {
            confirm(tl("trans.newvalues"));
            document.location.reload();
        }

        setTimeout(poll_counters, COUNTER_POLL_INTERVAL_MS);
    });
}

iListScroll = null;

function InitTransData() {
    $.get("/lib/transients/transient_info_jasic.html", DataReceive);
}

function ById(id) {
    return document.getElementById(id);
}

function addStyleClass(elem, cssClassName) {
    if (elem.className.indexOf(cssClassName) < 0)
        elem.className += " " + cssClassName;
}

function removeStyleClass(elem, cssClassName) {
    elem.className = elem.className.replace(cssClassName, "");
}

const setCheckboxForChannel = (channel, value) => {
    if (channel % 2 === 0) {
        $('#IL' + (channel / 2)).prop("checked", value);
        $('#colorboxIL' + (channel / 2)).css("visibility", value === true ? "visible" : "hidden");
    } else {
        $('#UL' + ((channel + 1) / 2)).prop("checked", value);
        $('#colorboxUL' + ((channel + 1) / 2)).css("visibility", value === true ? "visible" : "hidden");
    }
};

const resetChannelCheckboxes = () => {
    $('#UL1').prop("checked", false);
    $('#UL2').prop("checked", false);
    $('#UL3').prop("checked", false);
    $('#UL4').prop("checked", false);
    $('#IL1').prop("checked", false);
    $('#IL2').prop("checked", false);
    $('#IL3').prop("checked", false);
    $('#IL4').prop("checked", false);
    $('#colorboxUL1').css("visibility", "hidden");
    $('#colorboxUL2').css("visibility", "hidden");
    $('#colorboxUL3').css("visibility", "hidden");
    $('#colorboxUL4').css("visibility", "hidden");
    $('#colorboxIL1').css("visibility", "hidden");
    $('#colorboxIL2').css("visibility", "hidden");
    $('#colorboxIL3').css("visibility", "hidden");
    $('#colorboxIL4').css("visibility", "hidden");
};

const enableDIsableCheckboxes = (enabled) => {
    if (enabled) {
        $('#UL1').removeAttr("disabled");
        $('#UL2').removeAttr("disabled");
        $('#UL3').removeAttr("disabled");
        $('#UL4').removeAttr("disabled");
        $('#IL1').removeAttr("disabled");
        $('#IL2').removeAttr("disabled");
        $('#IL3').removeAttr("disabled");
        $('#IL4').removeAttr("disabled");
    } else {
        $('#UL1').attr("disabled", true);
        $('#UL2').attr("disabled", true);
        $('#UL3').attr("disabled", true);
        $('#UL4').attr("disabled", true);
        $('#IL1').attr("disabled", true);
        $('#IL2').attr("disabled", true);
        $('#IL3').attr("disabled", true);
        $('#IL4').attr("disabled", true);
    }
};

setTimeout(function () {
    InitTransData();
}, 200);

setTimeout(function () {
    poll_counters();
}, COUNTER_POLL_INTERVAL_MS);

const bittest = (number, nth) => (number & (1 << nth)) != 0;

// Exporting function for unit tests
module.exports = {
    mapFlagsToReasonOfTransient, bittest
};
