const GRAPH_OPTIONS = {
    linewidth: 1,
    xaxis: {
        tickFormatter: function (val, axis) {
            return dateFormat(getFakedLocalDeviceTime(0.001 * val), "HH:MM:ss.l");
        },
        autoscale: false
    },
    yaxis: {
        tickFormatter: (val) => val + " V",
        autoscale: true
    },
    y2axis: {
        tickFormatter: function (val, axis) {
            return val + " A";
        },
        autoscale: true
    },
    grid: {
        minorVerticalLines: true
    },
    mouse: {
        track: true,
        relative: true,
        sensibility: 10,
        radius: 5,
        trackFormatter: function (obj) {
            var time = getFakedLocalDeviceTime(0.001 * obj.nearest.x);
            var unit = "";
            if (obj.series.yaxis.n === 1) {
                unit = " V";
            } else {
                unit = " A";
            }

            return obj.y + unit + ", " + dateFormat(time, "HH:MM:ss.l");
        }
    },
    selection: {
        mode: 'x',
        fps: 30
    },
    legend: {
        position: 'se'
    },
    colors: ['#FF0000', '#0000FF', '#FFF000', '#229954', '#8E44AD', '#DC7633', '#85C1E9', '#95A5A6']
};

const CHANNELS = Object.freeze({"UL1": 1, "UL2": 3, "UL3": 5, "UL4": 7, "IL1": 2, "IL2": 4, "IL3": 6, "IL4": 8});

class TransientGraph {
    constructor(containerId) {
        this.graphData = {};
        this.container = document.getElementById(containerId);
        this.additionalOptions = {};
        Flotr.EventAdapter.observe(this.container, 'flotr:select', (area) => {
            // Draw graph with new area
            this.additionalOptions = {
                xaxis: {
                    min: area.x1,
                    max: area.x2,
                    tickFormatter: function (val, axis) {
                        return dateFormat(getFakedLocalDeviceTime(0.001 * val), "HH:MM:ss.l");
                    },
                    autoscale: false
                }
            };
            this.redraw();
        });
    }

    clearData = () => {
        for (let i = 0; i < 8; i++)
            this.graphData[i] = {};
    };

    resetZoomState = () => {
        this.additionalOptions["xaxis"] = {
            tickFormatter: function (val, axis) {
                return dateFormat(getFakedLocalDeviceTime(0.001 * val), "HH:MM:ss.l");
            },
            autoscale: false
        }
    };

    clearDataForChannel = (channel) => {
        this.graphData[channel - 1] = {};
    };

    addDataForChannel = (channel, transientData, startTime, freq, pre) => {
        const data = [];
        for (let i = 0; i < transientData.length; i++) {
            const timeStamp = startTime + (1000 / freq * (i - pre));
            data.push([timeStamp, transientData[i]]);
        }
        this.graphData[channel - 1] = {
            data: data,
            yaxis: channel % 2 === 0 ? 2 : 1
        };
    };

    redraw = () => {
        // Clone the options, so the 'options' variable always keeps intact.
        this.editYAxisOoptions(this.graphData);
        const options = Flotr._.extend(Flotr._.clone(GRAPH_OPTIONS), this.additionalOptions);

        // Draw a new graph.
        Flotr.draw(this.container, this.graphData, options);
    };

    editYAxisOoptions = (data) => {
        const maxValueYAxis1 = 1.01 * this.maxValueForYAxis(data, 1);
        const maxValueYAxis2 = 1.01 * this.maxValueForYAxis(data, 2);
        this.additionalOptions["yaxis"] = {
            tickFormatter: (val) => val + " V",
            autoscale: false,
            min: -maxValueYAxis1,
            max: maxValueYAxis1
        };
        this.additionalOptions["y2axis"] = {
            tickFormatter: (val) => val + " A",
            autoscale: false,
            min: -maxValueYAxis2,
            max: maxValueYAxis2
        };
    };

    maxValueForYAxis = (multipleGraphs, yAxis) => {
        return Object.values(multipleGraphs)
            .map(singleGraph => this.singleGraphMax(singleGraph, yAxis))
            .reduce((highestValue, currentValue) => currentValue > highestValue ? currentValue : highestValue, 0);
    };

    singleGraphMax = (singleGraph, yAxis) => {
        if (singleGraph["yaxis"] === yAxis)
            return singleGraph["data"]
                .map(pair => Math.abs(pair[1]))
                .reduce((highestValue, currentValue) => currentValue > highestValue ? currentValue : highestValue, 0);
        return 0;
    }
}